#	Programmer:	Daniel Pozmanter
#	E-mail:		drpython@bluebottle.com
#	Note:		You must reply to the verification e-mail to get through.
#
#	Copyright 2003-2004 Daniel Pozmanter
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    DrPython is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	
#Pop Up Menu Dialog

import os.path
import wx
import drScrolledMessageDialog
import drPopUp
import drShortcuts, drShortcutsFile

class drPopUpMenuDialog(wx.Dialog):

	def __init__(self, parent):
		wx.Dialog.__init__(self, parent, -1, ("Customize Pop Up Menu"), wx.Point(50, 50), wx.Size(550, 350), wx.DEFAULT_DIALOG_STYLE | wx.THICK_FRAME)
						
		wx.Yield()
		
		self.ID_PROGRAM = 1001
		self.ID_POPUP = 1002
		
		self.ID_LIST = 1300
		
		self.ID_ADD = 1003
		self.ID_REMOVE = 1004
		self.ID_UPDATE = 1005
		self.ID_SAVE = 1006
		
		self.ID_UP = 1111
		self.ID_DOWN = 2222
				
		self.theSizer = wx.FlexGridSizer(5, 4, 5, 10)
		self.menubuttonSizer = wx.BoxSizer(wx.VERTICAL)
		self.listSizer = wx.BoxSizer(wx.HORIZONTAL)
		
		self.homedirectory = parent.homedirectory
		
		popupmenulist = []
						
		map(popupmenulist.append, parent.popupmenulist)
		
		if len(popupmenulist) < 1:
				popupmenulist = ["<ROOT>", "Undo", "Redo", "<Separator>", "Cut", "Copy", "Paste", "Delete", "<Separator>", "Select All"]
		else:
			popupmenulist.insert(0, "<ROOT>")		
		
		self.programmenulist = drShortcutsFile.GetShortcutList()
		
		self.programmenulist.sort()		

		#STC
		
		self.stclist = []
		map(self.stclist.append, drShortcutsFile.GetSTCShortcutList())
		self.stclist.insert(0, "<Insert Separator>")
		
		#DrScript
		
		self.drscriptlist = []
		map(self.drscriptlist.append, parent.drscriptmenu.titles)
		x = 0
		l = len(self.drscriptlist)
		while x < l:
			self.drscriptlist[x] = "<DrScript>:" + self.drscriptlist[x]
			x = x + 1
		self.drscriptlist.insert(0, "<Insert Separator>")
		
		#Plugins
		plist = os.listdir(parent.homedirectory + "/plugins")
		
		self.PluginList = []
		for p in plist:
			i = p.find(".py")			
			l = len(p)
			if i > -1 and (i + 3 == l):			
				self.PluginList.append("<Plugin>:" + p[:i])
		
		list = ["Standard", "Text Control", "DrScript"]
		list.extend(self.PluginList)
				
		self.cboList = wx.ComboBox(self, self.ID_LIST, "Standard", wx.DefaultPosition, (200, -1), list, wx.CB_DROPDOWN)
		
		self.programmenu = wx.ListBox(self, self.ID_PROGRAM, wx.DefaultPosition, wx.Size(200, -1), self.programmenulist)
		
		self.popupmenu = wx.ListBox(self, self.ID_POPUP, wx.DefaultPosition, wx.Size(200, -1), popupmenulist)
		
		self.btnUp = wx.Button(self, self.ID_UP, " Up ")
		self.btnAdd = wx.Button(self, self.ID_ADD, " ---> ")
		self.btnRemove = wx.Button(self, self.ID_REMOVE, " Remove ")
		self.btnDown = wx.Button(self, self.ID_DOWN, " Down ")
		
		self.menubuttonSizer.Add(wx.StaticText(self, -1, "   "), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.menubuttonSizer.Add(self.btnAdd, 0, wx.SHAPED)
		self.menubuttonSizer.Add(wx.StaticText(self, -1, "   "), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.menubuttonSizer.Add(wx.StaticText(self, -1, "   "), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.menubuttonSizer.Add(self.btnUp, 0, wx.SHAPED)
		self.menubuttonSizer.Add(wx.StaticText(self, -1, "   "), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.menubuttonSizer.Add(self.btnDown, 0, wx.SHAPED)
		self.menubuttonSizer.Add(wx.StaticText(self, -1, "   "), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.menubuttonSizer.Add(wx.StaticText(self, -1, "   "), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.menubuttonSizer.Add(self.btnRemove, 0, wx.SHAPED)
		
		self.theSizer.Add(wx.StaticText(self, -1, ""), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 0, wx.ALIGN_CENTER | wx.SHAPED)
		
		self.listSizer.Add(wx.StaticText(self, -1, "List: "), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.listSizer.Add(self.cboList, 0, wx.ALIGN_CENTER | wx.SHAPED)
		
		self.theSizer.Add(wx.StaticText(self, -1, ""), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.theSizer.Add(self.listSizer, 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 0, wx.ALIGN_CENTER | wx.SHAPED)	
		
		self.theSizer.Add(wx.StaticText(self, -1, ""), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Current List:"), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Pop Up Menu:"), 0, wx.ALIGN_CENTER | wx.SHAPED)
		
		self.theSizer.Add(wx.StaticText(self, -1, ""), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.theSizer.Add(self.programmenu, 0, wx.SHAPED | wx.ALIGN_CENTER)
		self.theSizer.Add(self.menubuttonSizer, 0, wx.SHAPED | wx.ALIGN_CENTER)
		self.theSizer.Add(self.popupmenu, 0,  wx.SHAPED | wx.ALIGN_CENTER)
		
		self.parent = parent
								
		self.theSizer.Add(wx.StaticText(self, -1, ""), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, ""), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, ""), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, ""), 0, wx.ALIGN_CENTER | wx.SHAPED)
				
		self.btnUpdate = wx.Button(self, self.ID_UPDATE, "&Update")
		self.btnSave = wx.Button(self, self.ID_SAVE, "&Save")
		
		self.btnClose = wx.Button(self, 101, "&Close")
		self.theSizer.Add(wx.StaticText(self, -1, ""), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.theSizer.Add(self.btnClose, 0,  wx.SHAPED | wx.ALIGN_CENTER)
		self.theSizer.Add(self.btnUpdate, 0,  wx.SHAPED | wx.ALIGN_CENTER)
		self.theSizer.Add(self.btnSave, 0,  wx.SHAPED | wx.ALIGN_CENTER)
		self.btnClose.SetDefault()

		self.SetAutoLayout(True)
		self.SetSizer(self.theSizer)
	
		self.Bind(wx.EVT_BUTTON, self.OnbtnUp, id=self.ID_UP)
		self.Bind(wx.EVT_BUTTON, self.OnbtnAdd, id=self.ID_ADD)
		self.Bind(wx.EVT_BUTTON, self.OnbtnRemove, id=self.ID_REMOVE)
		self.Bind(wx.EVT_BUTTON, self.OnbtnDown, id=self.ID_DOWN)
		self.Bind(wx.EVT_BUTTON, self.OnbtnUpdate, id=self.ID_UPDATE)
		self.Bind(wx.EVT_BUTTON, self.OnbtnSave, id=self.ID_SAVE)
		self.Bind(wx.EVT_BUTTON, self.OnbtnClose, id=101)
								
		self.Bind(wx.EVT_COMBOBOX, self.OnList, id=self.ID_LIST)
				
	def OnbtnAdd(self, event):	
		tselection = self.programmenu.GetStringSelection()
		tsel = self.programmenu.GetSelection()
		if tsel == -1:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, "Nothing Selected to Add", "Mistake")
			d.ShowModal()
			d.Destroy()
			return	
			
		sel = self.popupmenu.GetSelection()
		if sel == -1:
			sel = 0
			
		separator = (tselection == "<Insert Separator>")
		if separator:
			tselection = "<Separator>"

		self.popupmenu.InsertItems([tselection], sel+1)
		self.popupmenu.SetSelection(sel+1)
	
	def OnbtnClose(self, event):
		self.Close(1)
		
	def OnbtnDown(self, event):
		sel = self.popupmenu.GetSelection()
		if sel < self.popupmenu.GetCount()-1 and sel > 0:
			txt = self.popupmenu.GetString(sel)
			self.popupmenu.Delete(sel)
			self.popupmenu.InsertItems([txt], sel+1)
			self.popupmenu.SetSelection(sel+1)
	
	def OnbtnRemove(self, event):
		sel = self.popupmenu.GetSelection()
		selection = self.popupmenu.GetStringSelection()		
		if not sel:			
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, "You cannot remove the root item.", "Mistake")
			d.ShowModal()
			d.Destroy()
			return
		if sel == -1:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, "Nothing Selected to Remove", "Mistake")
			d.ShowModal()
			d.Destroy()
			return

		self.popupmenu.Delete(sel)
		self.popupmenu.SetSelection(sel-1)
	
	def OnbtnUp(self, event):
		sel = self.popupmenu.GetSelection()		
		if sel > 1:
			txt = self.popupmenu.GetString(sel)
			self.popupmenu.Delete(sel)
			self.popupmenu.InsertItems([txt], sel-1)
			self.popupmenu.SetSelection(sel-1)
	
	def OnbtnUpdate(self, event):
		y = 0
		c = self.popupmenu.GetCount()
		
		popupmenulist = []
		
	
		while y < c:
			pop = self.popupmenu.GetString(y)
			if not pop == "<ROOT>":		
				popupmenulist.append(pop)
			y = y + 1
		
		self.parent.popupmenulist = popupmenulist
					
		if self.parent.prefs.enablefeedback:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Succesfully updated the current instance of DrPython.\nClick Save to make it permanent."), "Updated Pop Up Menu")
			d.ShowModal()
			d.Destroy()
		
	def OnbtnSave(self, event):
		y = 0
		c = self.popupmenu.GetCount()
		
		popupmenustring = ""
		popupmenulist = []
				
		while y < c:
			pop = self.popupmenu.GetString(y)
			if not pop == "<ROOT>":
				popupmenustring = popupmenustring + pop + "\n"
				popupmenulist.append(pop)
			y = y + 1			
						
		self.parent.popupmenulist = popupmenulist
	
		if (not os.path.exists(self.homedirectory)):
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Dude, you've got some problems...\nYour homedirectory (" + self.homedirectory + ") does not exist!\nLet's not bother speculating about how or why.\nRead the help file for this truly screwed up situation.\nDrPython will now politely ignore your request to save.\nTry again when you have fixed this problem."), "Huge Error")
			d.ShowModal()
			d.Destroy()
			return
		popupfile = self.homedirectory + "/popupmenu.dat"
		try:			
			f = file(popupfile, 'w')
			f.write(popupmenustring)
			f.close()	
		except IOError:			
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("There were some problems writing to:\n"  + popupfile + "\nEither the file is having metaphysical issues, or you do not have permission to write.\nFor metaphysical issues, consult the documentation.\nFor permission issues, change the permissions on the directory to allow yourself write access.\nDrPython will now politely ignore your request to save.\nTry again when you have fixed the problem."), "Write Error")
			d.ShowModal()
			d.Destroy()
			return
		if self.parent.prefs.enablefeedback:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Succesfully wrote to:\n"  + popupfile + "\nand updated the current instance of DrPython."), "Saved Pop Up Menu")
			d.ShowModal()
			d.Destroy()
			
	def OnList(self, event):
		sel = self.cboList.GetSelection()
			
		if sel == 0:
			self.programmenu.Set(self.programmenulist)
		elif sel == 1:
			self.programmenu.Set(self.stclist)
		elif sel == 2:
			self.programmenu.Set(self.drscriptlist)
		else:
			pluginlist = []
			seltext = self.cboList.GetStringSelection()
			if seltext.find("<Plugin>") > -1:
				seltext = seltext[seltext.find(':')+1:]
				x = 0
				l = len(self.parent.PluginPopUpMenuNames)
				while x < l:
					if self.parent.PluginPopUpMenuNames[x] == seltext:
						pluginlist.append("<Plugin>:"+self.parent.PluginPopUpMenuLabels[x])
					x = x + 1
			pluginlist.insert(0, "<Insert Separator>")
			self.programmenu.Set(pluginlist)
				
				